function results=maxent_int(N,side,cutoff,momin,momout,lam0,interval,tol,maxit,ref,w,mm)
%----------------------------------------------------------------------------
%fit moment conditions to the maxent density based on interval data
%results=maxent3(side,cutoff,momin,[momout,lam0,incrt,tol,maxit,ref,w])
%----------------------------------------------------------------------------
%N:		sample size
%side:          the side condition. k by j matrix.
%                       For k=1, fit the moment conditions of the to maxent density on the range specified by cutoff. 
%           The first entry is usually one and can be the share of the specified range.          
%                       For k>1, k is the number of groups and j is the order of moments. The first column is the
%zeroth moment,
%                       or share of each group; the second, the conditional mean; the third, the conditional second
%moment and so on.
%cutoff:        cutoff points for the group. The numerical integration is conducted on the interval 
%                       specified within the two endpoints of cutoff.
%momin:         description of the side condition. No need to specify the first column, which is default to 
%                       be the zeroth moment. The input should be a string of the form, say, '[x log(x) x.^2 x.^3]'.
%The inputs in square bracket are optional.
%momout:        description of the characterizing moments of fitted density, default to be the same as input moments.
%                       No need to specify the constant. The specification of the model can be different from input
%moments.    
%                       The number of characterizing moments should not be greater than the number of moment
%constraints k*j.
%lam0:          initial value.
%interval:      number of the equal-spaced intervals for range x. Default is 5,000.
%tol:           termination tolerance. Default is 1e-6.
%maxit:         maximum number of iterations. Default is 100.
%ref:           The reference distribution for minimization of cross-entropy. ref enters as the empirical distribution
%                       function and takes the same number of elements as the number of points to be evaluated on the
%integration range.
%w:                     weighting matrix.
%-----------------------------------------------------------------------------
%results.lambda:        Lagrange multiplier.
%results.density:       Predicted y given the interval x and increment.
%results.jacobian:      The Jocobian matrix.
%results.iter:      Number of iterations
%results.entropy:   Maximized entropy (only meaningful in the classical case, ie., identical number and form of
%                   input and output moments).
%-----------------------------------------------------------------------------
%Author: Ximing Wu; Date: 3/11/2002.

%make two changes compared to previous version: 
%(see "A Matlab Program to Calculate the Maximum Entropy Distributions" by Ali Mohammad-Djafari).
%1. set the initial values as that of uniform distribution; this initial value is more likely to
%   lead to convergence since it is value for a distribution, moreover, the distribution to which desired density
%   is the closest.
%2. use simplified Simposon's mehtod, which is faster than Matlab's trapz function.

%Ximing Wu; Date: 7/2/2004
%add variance-covariance matrix
%only valid when for just identification cases, ie. when k=1
% For details, see Wu, X and Perloff, J. GMM Estimation of Maximum Entropy Density with Interval Data, 
% Journal of Econometrics, 138 (2007): 532-546.


%Initialization
interv=5000;
k=size(side,1);
j=size(side,2);
Tol=1e-6;
Maxit=100;

if (k+1)~=length(cutoff)
        error('The number of cutoff points is not consistent with the number of groups in moment constraints.')
end

if nargin>6&~isempty(interval)
        interv=interval;
end
if nargin>7&~isempty(tol)
        Tol=tol;
end
if nargin>8&~isempty(maxit)
        Maxit=maxit;
end
increment=range(cutoff)/(interv-1);
x=cutoff(1):increment:cutoff(length(cutoff));
x=x';

xx1=ones(length(x),1);
temp=inline(momin);
temp=temp(x);
if sum((xx1-temp(:,1)).^2)~=0
        xx1=[xx1 temp];
end
if size(xx1,2)~=j
        error('The number of input moments is different from the number of moment constraints.')
end

if nargin>4&~isempty(momout)
    xx2=ones(length(x),1);
        temp=inline(momout,'x');
        temp=temp(x);
        xx2=[xx2 temp];
else
        xx2=xx1;
        momout=momin;
end         
if size(xx2,2)-1>(k*j)
        error('The number of output moments is greater than the number of input moments.')
end

n=size(xx2,2);
lambda=zeros(n,1);
% set a uniform distribution for initial values
lambda(1)=log(range(x));
if nargin>5&~isempty(lam0)
        lambda=lam0;
else
        lam0=[];
end

for i=1:(j-1)
        side(:,i+1)=side(:,i+1).*side(:,1);
end

side=reshape(side',[k*j 1]);
lambda=reshape(lambda,[length(lambda) 1]);
cutoff=reshape(cutoff,[length(cutoff) 1]);

tot=1;
t=0;
tol=1e-10;
change=1;
while change>Tol&t<Maxit
    t=t+1;
    y=exp(-xx2(:,1:length(lambda))*lambda);
    %y=exp(-xx2(:,1:length(lambda))*lambda);
    if nargin>9&~isempty(ref)
                y=y.*ref;
        else
        ref=[];
        end

    jac=zeros(k*j,n);
    i=1;
    while i<=length(side)/j
        index=find(x>=cutoff(i)&x<=cutoff(i+1));
        for q=1:j
            %Use the simplified Simposon's method. Since the integration is implemented using
            %equal-space increment, this method is essential identical to trapz function.
            temp=increment*sum(xx2(index,1:n).*repmat(y(index,:).*xx1(index,q),[1 n]));
                %temp=trapz(x(index),xx2(index,1:n).*repmat(y(index,:).*xx1(index,q),[1 n]));
                jac(j*i-j+q,:)=temp;
        end
        i=i+1;
    end

    b=side-jac(:,1);
    if nargin>10&~isempty(w)
        incr=inv(jac'*w*jac)*jac'*w*b;
    else    
        incr=jac\b;
        w=[];
    end    
    tot0=tot;
    tot=b'*jac*incr;
        lambda=lambda-incr;
        %change=abs(tot-tot0);
        change=sum(abs(b))/length(b);
    
end

iter=t;

if change>Tol|change==0|isnan(change)
        lambda=zeros(n,1);
end

%rescale so the density integrates to unity
t=exp(-xx2(:,2:length(lambda))*lambda(2:end));
z=log(sum(t)*increment);
lambda(1)=z;
y=exp(-xx2(:,1:length(lambda))*lambda);

    if nargin>9&~isempty(ref)
                y=y.*ref;
        else
        ref=[];
        end

    jac=zeros(k*j,n);
    i=1;
    while i<=length(side)/j
        index=find(x>=cutoff(i)&x<=cutoff(i+1));
        for q=1:j
            %Use the simplified Simposon's method. Since the integration is implemented using
            %equal-space increment, this method is essential identical to trapz function.
            temp=increment*sum(xx2(index,1:n).*repmat(y(index,:).*xx1(index,q),[1 n]));
                %temp=trapz(x(index),xx2(index,1:n).*repmat(y(index,:).*xx1(index,q),[1 n]));
                jac(j*i-j+q,:)=temp;
        end
        i=i+1;
    end

%variance-covariance; only valid when for just identification cases, ie. when k=1
v=zeros(j-1,j-1);
for i=1:(j-1)
	for ii=1:(j-1)
		v(i,ii)=jac(i+1,ii+1)-jac(1,i+1)*jac(1,ii+1);
	end
end	
v=inv(v)/N;

results.density=y;
results.lambda=lambda;
results.entropy=jac(1,:)*lambda;
results.jacobian=jac;
results.cov=v;
results.iter=iter;
results.side=side;
results.cutoff=cutoff;
results.momin=momin;
results.momout=momout;
results.interval=interv;
results.lam0=lam0;
results.ref=ref;
results.w=w;
%results.t=t;